﻿using System;
using System.Net;
using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.VersionControl.Client;

namespace TfsBranchSearching
{
    internal class Program
    {
        private static readonly Logger _log = new Logger(Console.Out);
        private static VersionControlServer _vcs;

        private static void Main(string[] args)
        {
            var uri = new Uri("http://tfs:8080/tfs");
            var credentials = new NetworkCredential("user", "password", "domain");
            var projCollection = new TfsTeamProjectCollection(uri, credentials);
            projCollection.EnsureAuthenticated();
            _log.WriteLine("Authenticated successfully");

            _vcs = projCollection.GetService<VersionControlServer>();

            string srcFolder = "$/Project/Folder";

            var specs = new[] {new ItemSpec(srcFolder, RecursionType.None)};
            using (_log.WriteBlock(string.Format("Source folder {0}:", srcFolder)))
            {
                BranchHistoryTreeItem[][] branchHistory = _vcs.GetBranchHistory(specs, VersionSpec.Latest);
                BranchHistoryTreeItem[] srcFolderHistory = branchHistory[0];
                foreach (BranchHistoryTreeItem item in srcFolderHistory)
                {
                    Item(item);
                }
            }
        }

        private static void Item(BranchHistoryTreeItem item)
        {
            BranchRelative relative = item.Relative;
            Item branchToItem = relative.BranchToItem;
            long branchFromChangeset = relative.BranchFromItem == null ? 0 : relative.BranchFromItem.ChangesetId;
            using (_log.WriteBlock(string.Format("Branched to {0} in the {1} changeset (source item last change is {2})", branchToItem.ServerItem, branchToItem.ChangesetId, branchFromChangeset)))
                foreach (BranchHistoryTreeItem child in item.Children)
                {
                    Item(child);
                }
        }
    }
    
    #region Logger
    public class Logger
    {
        private readonly TextWriter _streamWriter;
        private const string IdentSymbol = "   ";

        private int _identLevel;

        private class Identation : IDisposable
        {
            private readonly Logger _logger;

            public Identation(Logger logger)
            {
                _logger = logger;

                _logger.AddIdentation();
            }

            public void Dispose()
            {
                _logger.ReduceIdentation();
            }
        }

        public Logger(TextWriter streamWriter)
        {
            _streamWriter = streamWriter;
        }

        public void WriteLine(string format, params object[] args)
        {
            for (int i = 0; i < _identLevel; i++)
                _streamWriter.Write(IdentSymbol);

            if (args.Length == 0)
                _streamWriter.WriteLine(format);
            else
                _streamWriter.WriteLine(format, args);
            _streamWriter.Flush();
        }

        public IDisposable WriteBlock(string format, params object[] args)
        {
            WriteLine(format, args);
            return new Identation(this);
        }

        private void AddIdentation()
        {
            _identLevel++;
        }

        private void ReduceIdentation()
        {
            if (_identLevel == 0)
                throw new InvalidOperationException();
            _identLevel--;
        }
    }
    #endregion
}